#define RTPMAP_LABEL_LEN	9
extern 	CCHAR Sdp_cRtpmap[];

#define FMTP_LABEL_LEN		7
extern 	CCHAR Sdp_cFmtp[];

#define RTCP_LABEL_LEN	7
extern 	CCHAR Sdp_cRtcp[];

#define TELEVENT_LABEL_LEN	15
extern 	CCHAR Sdp_cTelEvent[];

#define V0_LABEL_LEN		3
extern 	CCHAR Sdp_cV0[];

extern 	CCHAR Sdp_cMAudio[];

extern 	CCHAR Sdp_cAudioMap[CODEC_NULL][5];

// Functions in p_sdp.c
void SdpAddAudioCode();
void SdpAddDtmfType(UCHAR iDtmfPayload);
void SdpAddDtmfFmtp(UCHAR iDtmfPayload);
void SdpAddPTime(UCHAR iPTime);
void SdpAddRtpMap(UCHAR iType);
void SdpAddRtpMapList();
UCHAR SdpAudioMatch(UCHAR iType);

